@@ -5,6 +5,9 @@
 /***************************************************
  * CONSTRUCTOR
  ***************************************************/
+/**
+ * Wait for limits of triangulation in axis x and y.
+ */
 Delaunator::Delaunator(const float xmin, const float xmax, 
                 const float ymin, const float ymax) : xmin(xmin), xmax(xmax), ymin(ymin), ymax(ymax) {
 // Creation of primitive mesh, with four points.
@@ -107,6 +110,9 @@ Delaunator::Delaunator(const float xmin, const float xmax,
 
 
 
+/**
+ * Frees all Faces, Vertices and Edges.
+ */
 Delaunator::~Delaunator() {
         // free's !
         for(Face* it : this->faces) {
@@ -128,12 +134,10 @@ Delaunator::~Delaunator() {
 /***************************************************
  * PUBLIC METHODS
  ***************************************************/
-
-/*
- * ADD VERTEX
+/**
+ * Add a new vertex to Mesh at given coordinates.
+ * Return address of that point in Mesh, or NULL if not found or out of bounds.
  */
-// Add a new vertex to Mesh at given coordinates.
-// Return address of that point in Mesh, or NULL if not found or out of bounds.
 Vertex* Delaunator::addVertexAt(Coordinates p) {
 // initialization
         Face* container = NULL; // container of p
@@ -198,9 +202,9 @@ Vertex* Delaunator::addVertexAt(Coordinates p) {
 
 #if DELAUNAY_CONDITION
 // apply delaunay restrictions
-        this->flipOn(f1);
-        this->flipOn(f2);
-        this->flipOn(f3);
+        this->applyDelaunayCondition(f1);
+        this->applyDelaunayCondition(f2);
+        this->applyDelaunayCondition(f3);
 #endif
 // ending
         }}
@@ -209,12 +213,11 @@ Vertex* Delaunator::addVertexAt(Coordinates p) {
 
 
 
-/*
- * MOVE VERTEX
+/**
+ * Move given vertex by given vector values.
+ * Modify the mesh in consequence.
+ * Vertex can't pass over the mesh bounds.
  */
-// Move given vertex by given vector values.
-// Modify the mesh in consequence.
-// Vertex can't pass over the mesh bounds.
 void Delaunator::moveVertex(Vertex* v, float vec_x, float vec_y) {
 // Move the vertex
         //v->x() += vec_x;
@@ -243,7 +246,7 @@ void Delaunator::moveVertex(Vertex* v, float vec_x, float vec_y) {
 #if DEBUG
                 assert(f != NULL);
 #endif
-                this->flipOn(f);
+                this->applyDelaunayCondition(f);
         }
 #endif 
 }
@@ -252,11 +255,10 @@ void Delaunator::moveVertex(Vertex* v, float vec_x, float vec_y) {
 
 
 
-/*
- * VERTEX AT
+/**
+ * Return vertex found at given coords, around precision.
+ * Return NULL if no vertex found.
  */
-// Return vertex found at given coords, around precision.
-// Return NULL if no vertex found.
 Vertex* Delaunator::vertexAt(float x, float y, float precision) const {
         Vertex *target = NULL, *current = NULL;
         for(IteratorOnAllVertices_read it = this->allVertices_read(); target == NULL && it != it.end(); it++) {
@@ -264,22 +266,123 @@ Vertex* Delaunator::vertexAt(float x, float y, float precision) const {
                 if(fabs(x-current->x()) < (precision+EPSILON) && fabs(y-current->y()) < (precision+EPSILON))
                         target = current;
         }
-        //if(target != NULL) {
-                //logs("TARGET: %u, have %u neighbors.\n", target->getID(), target->neighbourCount());
-        //}
         return target;
 }
 
 
 
 
-
-/*
- * COLLIDE AT
+/**
+ * Remove a Vertex from the triangulation.
+ * Iterators will be invalidated, and vertex will be free.
+ * @param del_vrtx targeted Vertex
  */
-// Return true if given coordinates are in-limit of this.
-bool Delaunator::collideAt(Coordinates p) const {
-        return !(p.x() < this->xmin || this->xmax < p.x() || p.y() < this->ymin || this->ymax < p.y());
+void Delaunator::delVertex(Vertex* del_vrtx) {
+// INIT
+#if DEBUG
+        assert(del_vrtx != NULL);
+        assert(this->haveVertex(del_vrtx));
+        assert(!this->isCornerVertex(del_vrtx));
+#endif
+        bool modification = true; // false when no modification operate on triangulation
+        // Creat some container
+        std::vector<Face*> modified_faces(0); // modified faces that can break Delaunay condition
+        std::vector<Edge*> nei_edge(0);
+        std::vector<float> nei_dist(0);
+        std::vector<Vertex*> nei_vrtx(0);
+        // Fill those that refer neighbors, edge to neighbors or distance to neighbors
+        Edge* edge = del_vrtx->getEdge();
+        do {    nei_edge.push_back(edge);
+                nei_vrtx.push_back(edge->destinVertex());
+                nei_dist.push_back(edge->destinVertex()->distanceTo(*del_vrtx));
+                edge = edge->rotLeftEdge();
+        } while(edge != del_vrtx->getEdge());
+
+// SIMPLIFY LOCAL TRIANGULATION
+        // When we can't do something, it's when the del_vrtx is linked to tree points exactly.
+        // At this time of the algo, Delaunay condition is breaked, and suppression of the point
+        // is easy. 
+        while(modification) {
+                modification = false; 
+                // For each neighbour
+                for(unsigned int target = 0; target < nei_edge.size(); target++) {
+                        // take tree consecutivs neighbors indexes
+                        unsigned int id1 = target;
+                        unsigned int id2 = (target+1) % nei_dist.size();
+                        unsigned int id3 = (target+2) % nei_dist.size();
+                        // collision between del_vrtx and triangle formed by neighbors
+                        bool del_vrtx_in_triangle = geometry::pointInTriangle(
+                                        *nei_vrtx[id1], *nei_vrtx[id2], *nei_vrtx[id3], *del_vrtx
+                        );
+                        // if middle point is farthest than at least one of the two others from del_vrtx
+                        //      AND del_vrtx is not in triangle formed by the tree neighbors
+                        if((nei_dist[id2] >= nei_dist[id1] || nei_dist[id2] >= nei_dist[id3]) 
+                                        && !del_vrtx_in_triangle) {
+                                // these tree points will be a triangle !
+                                // so, operate flip on the middle edge
+                                this->operateFlip(nei_edge[id2]);
+                                modification  = true;
+                                // remove middle vertex (id2) from neighbors lists
+                                nei_edge.erase(nei_edge.begin() + id2);
+                                nei_dist.erase(nei_dist.begin() + id2);
+                                nei_vrtx.erase(nei_vrtx.begin() + id2);
+                                // The modified faces must be referenced : maybe it break the Delaunay condition
+                                modified_faces.push_back(nei_edge[id1]->nextLeftEdge()->rightFace());
+                        }
+                }
+        }
+        
+// DELETE POINT FROM TRIANGLE CONTAINER
+        // Container of del_vrtx is composed by the tree neighbors contains in nei_vrtx
+        // Delete del_vrtx is exactly the reverse of adding, after find the container.
+        Edge *edge1 = nei_edge.front();
+        Edge *edge2 = edge1->rotLeftEdge();
+        Edge *edge3 = edge2->rotLeftEdge();
+        // f2 and f3 will be deleted. f1 is the future Face of triangle
+        Face *f1 = edge1->leftFace(), *f2 = edge2->leftFace(), *f3 = edge3->leftFace();
+        // sides are the edge that are the sides of the final triangle of face f1
+        Edge *side1l = edge1->nextLeftEdge();
+        Edge *side2l = edge2->nextLeftEdge();
+        Edge *side3l = edge3->nextLeftEdge();
+#if DEBUG
+        // Some tests
+        assert(nei_vrtx.size() == 3);
+        assert(edge3  != edge1  && edge1  != edge2  && edge2  != edge3 );
+        assert(side3l != side1l && side1l != side2l && side2l != side3l);
+        assert(f1 != f3 && f3 != f2 && f2 != f1);
+        this->DEBUG_tests();
+#endif
+        // Origin Vertices must refers sides, no edge1, edge2 or edge3
+        side1l->originVertex()->setEdge(side1l);
+        side2l->originVertex()->setEdge(side2l);
+        side3l->originVertex()->setEdge(side3l);
+        // Set next left edge
+        side1l->setNextLeftEdge(side2l);
+        side2l->setNextLeftEdge(side3l);
+        side3l->setNextLeftEdge(side1l);
+        // Set face's Edge reference to a valide side, and reference it as a modified face
+        f1->setEdge(side1l);
+        modified_faces.push_back(f1);
+        // Delete unwanted faces, edges, and finally del_vrtx
+        this->removeEdgeFromEdges(edge1->oppositeEdge());
+        this->removeEdgeFromEdges(edge2->oppositeEdge());
+        this->removeEdgeFromEdges(edge3->oppositeEdge());
+        this->removeEdgeFromEdges(edge1);
+        this->removeEdgeFromEdges(edge2);
+        this->removeEdgeFromEdges(edge3);
+        this->removeFaceFromFaces(f2);
+        this->removeFaceFromFaces(f3);
+        this->removeVertexFromVertices(del_vrtx);
+
+// RESTORE DELAUNAY CONDITION
+        // Delaunay condition was break. It's time to restore it.
+        for(Face* face : modified_faces) 
+                this->applyDelaunayCondition(face);
+
+// END
+#if DEBUG
+        this->DEBUG_tests();
+#endif
 }
 
 
@@ -287,14 +390,20 @@ bool Delaunator::collideAt(Coordinates p) const {
 
 #if DEBUG // some tests with assertions
 /*
- * DEBUG TESTS
+ * DEBUG TESTS.
  */
 void Delaunator::DEBUG_tests() const {
         for(IteratorOnAllEdges_read it = this->allEdges_read(); it != it.end(); it++) {
                 assert((*it)->originVertex() != NULL);
                 assert((*it)->oppositeEdge() != NULL);
                 assert((*it)->leftFace() != NULL);
-                assert((*it)->nextLeftEdge()->nextLeftEdge()->nextLeftEdge() == (*it));
+                if((*it)->nextLeftEdge()->nextLeftEdge()->nextLeftEdge() != (*it)) {
+                        logs("%u, ", (*it)->getID());
+                        logs("%u, ", (*it)->nextLeftEdge()->getID());
+                        logs("%u, ", (*it)->nextLeftEdge()->nextLeftEdge()->getID());
+                        logs("%u, ", (*it)->nextLeftEdge()->nextLeftEdge()->nextLeftEdge()->getID());
+                        assert((*it)->nextLeftEdge()->nextLeftEdge()->nextLeftEdge() == (*it));
+                }
         }
         for(IteratorOnAllFaces_read it = this->allFaces_read(); it != it.end(); it++) {
                 assert((*it)->getEdge() != NULL);
@@ -311,13 +420,84 @@ void Delaunator::DEBUG_tests() const {
 
 
 
+
+
+
+
+/***************************************************
+ * ACCESSORS
+ ***************************************************/
+/**
+ * Return index of a Vertex in Delaunator container.
+ * @param v Vertex that will be found
+ * @return an unsigned int that is the index of the Vertex for this Deulaunator, or equal to number of Vertice if not found
+ */
+unsigned int Delaunator::getIndexOf(Vertex* v) const {
+        unsigned int index = this->vertices.size();
+        for(unsigned int i = this->vertices.size() - 1; i >= 0 
+                        && index == this->vertices.size(); i--) 
+                if(this->vertices[i] == v)
+                        index = i;
+        return index;
+}
+
+
+
+
+
+
+
+
+
+
+/***************************************************
+ * PREDICATS
+ ***************************************************/
+/**
+ * @param v tested Vertex 
+ * @return true iff tested Vertex is referenced by triangulation
+ */
+bool Delaunator::haveVertex(Vertex* v) const {
+        bool have = false;
+        for(unsigned int i = this->vertices.size() - 1; i >= 0 && not have; i--) 
+                have = (this->vertices[i] == v);
+        return have;
+}
+
+
+
+
+/**
+ * @param v tested Vertex 
+ * @return true iff tested Vertex is referenced by triangulation and is one of the four corner vertice
+ */
+bool Delaunator::isCornerVertex(Vertex* v) const {
+        return this->getIndexOf(v) < 4;
+}
+
+
+
+
+/**
+ * @param c Coordinates of tested point
+ * @return true if point is in-limit of this
+ */
+bool Delaunator::collideAt(Coordinates c) const {
+        return !(c.x() < this->xmin || this->xmax < c.x() || c.y() < this->ymin || this->ymax < c.y());
+}
+
+
+
+
+
 /***************************************************
  * ITERATORS
  ***************************************************/
-/*
- * VERTEX TO NEIGHBOUR VERTICES
+/**
+ * Infinite iteration on all neighbour vertices of a given Vertex.
+ * @param v the targeted Vertex
+ * @return an IteratorVertexToNeighbourVertices that iter on neighbors of target
  */
-// Infinite iteration on all neighbour vertices of a given Vertex.
 IteratorVertexToNeighbourVertices Delaunator::getNeighbors(Vertex* v) {
         return IteratorVertexToNeighbourVertices(v);
 }
@@ -329,12 +509,10 @@ IteratorVertexToNeighbourVertices Delaunator::getNeighbors(Vertex* v) {
 /***************************************************
  * PRIVATE METHODS
  ***************************************************/
-/*
- * FIND CONTAINER OF COORDINATES
+/**
+ * @param target Coordinates that must be different of NULL.
+ * @return address of Face of Delaunator that contains given coordinates, or NULL if error. If out of bounds, return the unvisible face that contain p
  */
-// Return address of Face of Delaunator that contains given coordinates.
-// Coordinates must be != NULL and contained in bounds of Delaunator.
-// Return NULL if error. If out of bounds, return the unvisible face that contain p.
 Face* Delaunator::findContainerOf(Coordinates target) const {
 // initialization
         Face* container = NULL;
@@ -413,41 +591,35 @@ Face* Delaunator::findContainerOf(Coordinates target) const {
 
 
 
-/*
- * FLIP ON
+/**
+ * Operate Delaunay Flip Algorithm on Face if necessary.
+ * Recursiv call on new triangles created by flip.
+ * @param f_ref a reference to a Face that can't be NULL and must be integrated in triangulation
+ * @param ttl time-to-live, or number of recursiv call operated
+ * @return true if modifications operate on tiangulation
  */
-// Operate Delaunay Flip Algorithm on Face if necessary.
-// Face can't be NULL and must be integrate in triangulation. (== have a valid edge field)
-// Return true if modifications.
-// Recursiv call on new triangles created by flip.
-bool Delaunator::flipOn(Face* f_ref, unsigned int ttl) {
 #if DEBUG
+bool Delaunator::applyDelaunayCondition(Face* f_ref, unsigned int ttl) {
         assert(f_ref != NULL && f_ref->getEdge() != NULL);
+#else 
+bool Delaunator::applyDelaunayCondition(Face* f_ref) {
 #endif
 // INITIALIZATION
         bool flip_done = true; // we are optimist.
-        Face* f_nei = NULL; // neighbour of f_ref that have an illegal side with it.
-        Edge* illegal_edge1 = NULL; // illegal edge that relie the illegal vertex and the f_ref's vertex.
-        Edge* illegal_edge2 = NULL; // and it's opposite.
-        Vertex* illegal_vertex1 = NULL; // vertex that is illegal because of illegal edge.
-        Vertex* illegal_vertex2 = NULL; // idem. these two vertices will be linked by edge after flip.
-
+        Edge* illegal_edge = NULL; // illegal edge that relie the illegal vertex and the f_ref's vertex.
 
 // CONDITION: in circumcircle
         // FIRST NEIGHBOUR
         if(f_ref->circumcircleContainCoords(*f_ref->getEdge1()->nextRightEdge()->destinVertex())) {
-                illegal_edge1 = f_ref->getEdge1();
-                illegal_vertex1 = f_ref->getP3();
+                illegal_edge = f_ref->getEdge1();
 
         // SECOND NEIGHBOUR
         } else if(f_ref->circumcircleContainCoords(*f_ref->getEdge2()->nextRightEdge()->destinVertex())) {
-                illegal_edge1 = f_ref->getEdge2();
-                illegal_vertex1 = f_ref->getP1();
+                illegal_edge = f_ref->getEdge2();
 
         // THIRD NEIGHBOUR
         } else if(f_ref->circumcircleContainCoords(*f_ref->getEdge3()->nextRightEdge()->destinVertex())) {
-                illegal_edge1 = f_ref->getEdge3();
-                illegal_vertex1 = f_ref->getP2();
+                illegal_edge = f_ref->getEdge3();
 
         // NO ILLEGAL SIDE
         } else  flip_done = false;
@@ -455,76 +627,20 @@ bool Delaunator::flipOn(Face* f_ref, unsigned int ttl) {
 
 // MODIFICATION OF FACES
         if(flip_done) {
-
-                // Deduce some shortcuts
-                illegal_edge2 = illegal_edge1->oppositeEdge();
-                illegal_vertex2 = illegal_edge1->nextRightEdge()->destinVertex();
-                f_nei = illegal_edge2->leftFace();
-                // previous and next edge in the cycle that form the Face.
-                Edge* edge1_prev = illegal_edge1->nextLeftEdge();
-                Edge* edge1_next = illegal_edge1->nextRightEdge()->oppositeEdge();
-                Edge* edge2_prev = illegal_edge2->nextLeftEdge();
-                Edge* edge2_next = illegal_edge2->nextRightEdge()->oppositeEdge();
-
-#if DEBUG
-                assert(illegal_edge1->originVertex() != illegal_vertex1);
-                assert(illegal_edge2->originVertex() != illegal_vertex1);
-                assert(illegal_edge1->originVertex() != illegal_vertex2);
-                assert(illegal_edge2->originVertex() != illegal_vertex2);
-#endif
-
-                // Origin vertex of illegal edges can't reference illegal edges anymore
-                illegal_edge1->originVertex()->setEdge(edge2_prev);
-                illegal_edge2->originVertex()->setEdge(edge1_prev);
-
-                // Modify next left and right edges of non-illegal edges
-                // NB: use of inter vars because change the next left edge will change the result of nextRightEdge.
-                edge1_prev->setNextLeftEdge(illegal_edge1);
-                edge2_prev->setNextLeftEdge(illegal_edge2);
-
-                edge1_next->setNextLeftEdge(edge1_prev);
-                edge2_next->setNextLeftEdge(edge2_prev);
-
-                illegal_edge1->setNextLeftEdge(edge1_next); // Close the cycle
-                illegal_edge2->setNextLeftEdge(edge2_next); 
-
-
-                // assign origin vertex
-                illegal_edge1->setOriginVertex(illegal_vertex1);
-                illegal_edge2->setOriginVertex(illegal_vertex2);
-
-                // update faces
-                f_ref->setEdge(illegal_edge1);
-                f_nei->setEdge(illegal_edge2);
-
-#if DEBUG
-                assert(illegal_edge1->originVertex() == illegal_vertex1);
-                assert(illegal_edge1->nextLeftEdge() == illegal_edge2->nextRightEdge()->nextRightEdge()->oppositeEdge());
-                assert(illegal_edge1->nextLeftEdge()->nextLeftEdge()->nextLeftEdge() == illegal_edge1);
-                assert(illegal_edge1->leftFace() == f_ref);
-                assert(illegal_edge1->nextLeftEdge()->leftFace() == f_ref);
-                assert(illegal_edge1->nextLeftEdge()->nextLeftEdge()->leftFace() == f_ref);
-
-                assert(illegal_edge2->destinVertex() == illegal_vertex1);
-                assert(illegal_edge2->nextLeftEdge()->nextLeftEdge()->nextLeftEdge() == illegal_edge2);
-                assert(illegal_edge2->leftFace() == f_nei);
-                assert(illegal_edge2->nextLeftEdge()->leftFace() == f_nei);
-                assert(illegal_edge2->nextLeftEdge()->nextLeftEdge()->leftFace() == f_nei);
-
-                this->DEBUG_tests();
-#endif
-#ifdef FOLLOW_SEARCH
-                illegal_edge1->passing= false;
-                illegal_edge2->passing= false;
-#endif
+                this->operateFlip(illegal_edge);
                 // Recursiv call on the updated faces
+#if DEBUG
                 // protection against infinite recursive call.
                 if(ttl < this->faces.size()/2) {
-                        this->flipOn(f_ref, ttl+1);
-                        this->flipOn(f_nei, ttl+1);
+                        this->applyDelaunayCondition(illegal_edge->leftFace(), ttl+1);
+                        this->applyDelaunayCondition(illegal_edge->rightFace(), ttl+1);
                 } else {
                         logs("TTL %u reached !\n", ttl);
                 }
+#else
+                this->applyDelaunayCondition(illegal_edge->leftFace());
+                this->applyDelaunayCondition(illegal_edge->rightFace());
+#endif
         }
 
 
@@ -536,6 +652,79 @@ bool Delaunator::flipOn(Face* f_ref, unsigned int ttl) {
 
 
 
+/**
+ * Operate the flip algorithm, as if the received Edge don't respect the Delaunay condition.
+ * @param illegal_edge1 an Edge that is defined illegal
+ */
+void Delaunator::operateFlip(Edge* illegal_edge1) {
+// DEDUCE SOME SHORTCUTS
+        Edge *illegal_edge2 = illegal_edge1->oppositeEdge();
+        Vertex *illegal_vertex1 = illegal_edge1->nextLeftEdge()->destinVertex();
+        Vertex *illegal_vertex2 = illegal_edge1->nextRightEdge()->destinVertex();
+        Face *face1 = illegal_edge2->rightFace();
+        Face *face2 = illegal_edge2->leftFace();
+        // previous and next edge in the cycle that form the Face.
+        Edge* edge1_prev = illegal_edge1->nextLeftEdge();
+        Edge* edge1_next = illegal_edge1->nextRightEdge()->oppositeEdge();
+        Edge* edge2_prev = illegal_edge2->nextLeftEdge();
+        Edge* edge2_next = illegal_edge2->nextRightEdge()->oppositeEdge();
+
+
+#if DEBUG
+// TESTS
+        assert(illegal_edge1->originVertex() != illegal_vertex1);
+        assert(illegal_edge2->originVertex() != illegal_vertex1);
+        assert(illegal_edge1->originVertex() != illegal_vertex2);
+        assert(illegal_edge2->originVertex() != illegal_vertex2);
+#endif
+
+// MODIFICATIONS
+        // Origin vertex of illegal edges can't reference illegal edges anymore
+        illegal_edge1->originVertex()->setEdge(edge2_prev);
+        illegal_edge2->originVertex()->setEdge(edge1_prev);
+
+        // Modify next left and right edges of non-illegal edges
+        // NB: use of inter vars because change the next left edge will change the result of nextRightEdge.
+        edge1_prev->setNextLeftEdge(illegal_edge1);
+        edge2_prev->setNextLeftEdge(illegal_edge2);
+
+        edge1_next->setNextLeftEdge(edge1_prev);
+        edge2_next->setNextLeftEdge(edge2_prev);
+
+        illegal_edge1->setNextLeftEdge(edge1_next); // Close the cycle
+        illegal_edge2->setNextLeftEdge(edge2_next); 
+
+
+        // assign origin vertex
+        illegal_edge1->setOriginVertex(illegal_vertex1);
+        illegal_edge2->setOriginVertex(illegal_vertex2);
+
+        // update faces
+        face1->setEdge(illegal_edge1);
+        face2->setEdge(illegal_edge2);
+
+#if DEBUG
+// TESTS
+        assert(illegal_edge1->originVertex() == illegal_vertex1);
+        assert(illegal_edge1->nextLeftEdge() == illegal_edge2->nextRightEdge()->nextRightEdge()->oppositeEdge());
+        assert(illegal_edge1->nextLeftEdge()->nextLeftEdge()->nextLeftEdge() == illegal_edge1);
+        assert(illegal_edge1->leftFace() == face1);
+        assert(illegal_edge1->nextLeftEdge()->leftFace() == face1);
+        assert(illegal_edge1->nextLeftEdge()->nextLeftEdge()->leftFace() == face1);
+
+        assert(illegal_edge2->destinVertex() == illegal_vertex1);
+        assert(illegal_edge2->nextLeftEdge()->nextLeftEdge()->nextLeftEdge() == illegal_edge2);
+        assert(illegal_edge2->leftFace() == face2);
+        assert(illegal_edge2->nextLeftEdge()->leftFace() == face2);
+        assert(illegal_edge2->nextLeftEdge()->nextLeftEdge()->leftFace() == face2);
+
+        this->DEBUG_tests();
+#endif
+#ifdef FOLLOW_SEARCH
+        illegal_edge1->passing= false;
+        illegal_edge2->passing= false;
+#endif
+}
 
 
 
